﻿---------------------------------------------------------------------------

local L = AceLibrary("AceLocale-2.2"):new("Assessment")
local SML = AceLibrary("SharedMedia-1.0")

-----------------------------------------------------------------------------

L:RegisterTranslations("enUS", function() return {
	["History"] = true,
	["Number of fights to keep detailed information for. Use 0 to keep all."] = true,
	["Pause"] = true,
	["Pause gathering combat data."] = true,

	["Reset"] = true,
	["Resets fight data. Does not reset saved fight data."] = true,
	["Reset All"] = true,
	["Reset all fight data, including saved fight data."] = true,

	["Data Options"] = true,
	["Data options."] = true,

    ["Enumerate Data"] = true,
    ["Enumerate Sparse Data"] = true,
    ["Enumerate Parameter List"] = true,
    ["FilterAll"] = "Show All",
	["FilterNonZero"] = "Show Non-Zero Values",

	["Fight Data List"] = true,
	
	["All fight data has been reset"] = true,
	["Fight data has been reset"] = true,

	["Player"] = true,
	["Player Color"] = true,
	["Select the color to use for Player colortype."] = true,
	
	["Gather OOC"] = true,
	["Enable gathering of data while not in combat."] = true,

	["Enter a name to save the fight data."] = true,

	["Erase Fight Data"] = true,
	["Erase specific fight data."] = true,
	["Erase %s"] = true,
	["Click to permanently delete the fight data named \"%s\""] = true,

	["FILTER_NONE"] = "Gather no data.",
	["FILTER_SELF"] = "Gather self data only.",
	["FILTER_ALL"]  = "Gather all data.",
} end)

L:RegisterTranslations("koKR", function() return {
	["History"] = "이력",
	["Number of fights to keep detailed information for. Use 0 to keep all."] = "상세화된 정보를 기록할 전투의 개수입니다. 모두 기록하려면 0을 사용하세요.",
	["Pause"] = "일시정지",
	["Pause gathering combat data."] = "전투 데이터 수집을 일시 정지합니다.",

	["Reset"] = "초기화",
	["Resets fight data. Does not reset saved fight data."] = "전투 데이터를 초기화합니다. 저장된 전투 데이터는 초기화 하지 않습니다.",
	["Reset All"] = "전체 초기화",
	["Reset all fight data, including saved fight data."] = "저장된 전투 데이터를 포함하여 모든 전투 데이터를 초기화합니다.",

	["Data Options"] = "데이터 설정",
	["Data options."] = "데이터 설정입니다.",

	["Enumerate Data"] = "데이터 나열",
	["Enumerate Parameter List"] = "매개변수 목록 나열",
	["FilterAll"] = "전체 보기",
	["FilterNonZero"] = "0이 아닌 값 보기",

	["Fight Data List"] = "전투 데이터 목록",
	
	["All fight data has been reset"] = "모든 전투 데이터가 초기화 되었습니다.",
	["Fight data has been reset"] = "전투 데이터가 초기화되었습니다.",

	["Player"] = "플레이어",
	["Player Color"] = "플레이어 색상",
	["Select the color to use for Player colortype."] = "플레이어에 사용할 색상을 선택합니다.",
	
	["Gather OOC"] = "OOC 수집",
	["Enable gathering of data while not in combat."] = "전투 중이 아닌 동안에 데이터의 수집을 사용합니다.",

	["Enter a name to save the fight data."] = "전투 데이터를 저장할 이름을 입력하세요.",

	["Erase Fight Data"] = "전투 데이터 삭제",
	["Erase specific fight data."] = "특정 전투 데이터를 삭제합니다.",
	["Erase %s"] = "%s 삭제",
	["Click to permanently delete the fight data named \"%s\""] = "\"%s\" 전투 데이터를 영구히 삭제하려면 클릭하세요.",

	["FILTER_NONE"] = "수집 데이터 없음.",
	["FILTER_SELF"] = "자신의 데이터만 수집.",
	["FILTER_ALL"]  = "전체 테이터 수집.",
} end)

-----------------------------------------------------------------------------

Assessment = AceLibrary("AceAddon-2.0"):new("AceEvent-2.0", "AceConsole-2.0", "AceDB-2.0", "AceModuleCore-2.0")
Assessment.Debug = (UnitName("player") == "Roartindon") and function(_, text) ChatFrame3:AddMessage(text) end or function() end
Assessment.version = "r"..("$Revision: 46024 $"):match("%d+")
Assessment:RegisterDB("AssessmentDB", "AssessmentDBPerCharacter")
Assessment:RegisterDefaults('char', { fightData = { } } )
Assessment:RegisterDefaults('profile', { 
	history				= 3,
	pause				= false,
	gatherOutOfCombat	= false,
	playerColor			= { 1, 0, 1 },
} )

-----------------------------------------------------------------------------

local combatTime
local fightData
local profile

-----------------------------------------------------------------------------

local dataTablePool = setmetatable({ }, { __mode="k" })

local select = select
local next = next
local pairs = pairs
local type = type

local function new()
	local recycledTable = next(dataTablePool)
	if recycledTable then
		dataTablePool[recycledTable] = nil
		return recycledTable
	else
		return { }
	end
end

local function del(t, recursive)
	for k,v in pairs(t) do
		if recursive and type(v) == "table" then
			del(v, true)
		end
		t[k] = nil
	end
	dataTablePool[t] = true
end

local function reset(t, recursive)
	for k,v in pairs(t) do
		if recursive and type(v) == "table" then
			del(v, true)
		end
		t[k] = nil
	end
end

local function copy(source)
	local result = new()

	for k,v in pairs(source) do
		if type(v) == "table" then
			result[k] = copy(v)
		else
			result[k] = v
		end
	end
	
	return result
end

Assessment.NewTable = new
Assessment.ReleaseTable = del
Assessment.ResetTable = reset
Assessment.CopyTable = copy

-----------------------------------------------------------------------------

local eraseFightDataArgs = { }
local eraseFightDataDirty = true
local eraseFightDataMetaTable = 
{ 
	__index = function(self, key)
				if key == "args" then
					if eraseFightDataDirty then
						Assessment:UpdateEraseFightData()
					end
					return eraseFightDataArgs
				end
			  end
}

Assessment.Options = 
{
	handler = Assessment,
	type = 'group',
	args = 
	{
		reset =
		{
			type = "execute",
			name = L["Reset"],
			desc = L["Resets fight data. Does not reset saved fight data."],
			func = function() Assessment:ResetUnnamedFightData() end,
			order = 1,
		},
		resetAll =
		{
			type = "execute",
			name = L["Reset All"],
			desc = L["Reset all fight data, including saved fight data."],
			func = function() Assessment:ResetFightData() end,
			order = 2,
		},
		eraseFightData = setmetatable(
		{
			type = "group",
			name = L["Erase Fight Data"],
			desc = L["Erase specific fight data."],
			disabled = function() return not next(eraseFightDataArgs) end,
			order = 3
		}, eraseFightDataMetaTable),
		history =
		{
			type = "range",
			name = L["History"],
			desc = L["Number of fights to keep detailed information for. Use 0 to keep all."],
			max = 20,
			min = 0,
			step = 1,
			get = function() return profile.history end,
			set = function(value) profile.history = value end,
			order = 10,
		},
		pause =
		{
			type = "toggle",
			name = L["Pause"],
			desc = L["Pause gathering combat data."],
			get = function() return profile.pause end,
			set = function(value) Assessment:SetPause(value) end,
			order = 10,
		},
		dataSpacer = 
		{
			type = 'header',
			order = 90,
		},
		data =
		{
			type = 'group',
			name = L["Data Options"],
			desc = L["Data options."],
			order = 100,
			args = 
			{
				playerColor =
				{
					type = "color",
					name = L["Player Color"],
					desc = L["Select the color to use for Player colortype."],
					get = function() return unpack(profile.playerColor) end,
					set = function(r, g, b) profile.playerColor = { r, g, b } Assessment:RedrawAll() end,
				},
				gatherOutOfCombat =
				{
					type = "toggle",
					name = L["Gather OOC"],
					desc = L["Enable gathering of data while not in combat."],
					get = function() return profile.gatherOutOfCombat end,
					set = function(v) profile.gatherOutOfCombat = v end,
				},
			},
		}
	},
}

local function EraseFightDataMethod(name) Assessment:EraseFightData(name) end
function Assessment:UpdateEraseFightData()
	if not fightData then return end
	reset(eraseFightDataArgs, true)
	eraseFightDataDirty = false
	
	for k in pairs(fightData) do
		if type(k) == "string" then
			local args = new()
			args.type = "execute"
			args.name = (L["Erase %s"]):format(k)
			args.desc = (L["Click to permanently delete the fight data named \"%s\""]):format(k)
			args.func = EraseFightDataMethod
			args.passValue = k
			eraseFightDataArgs[k] = args
		end
	end
end

-----------------------------------------------------------------------------

Assessment:RegisterChatCommand("/as", "/assessment", Assessment.Options)

-----------------------------------------------------------------------------

Assessment.ColorType			= { }
Assessment.DefaultDisplaySets	= { }
Assessment.DefaultTags			= { }
Assessment.FilterType			= { }
Assessment.ListType				= { }

-----------------------------------------------------------------------------

Assessment.EmptyMethod = function() end
local emptyMethod = Assessment.EmptyMethod

-----------------------------------------------------------------------------

local playerName = UnitName("player")
Assessment.ColorType.Player = 
{
	method = function(fightData, data)
				if data.key ~= playerName then return end
				return profile.playerColor
			 end,
	description = L["Player"]
}

Assessment.FilterType.All =
{
	tag			= "1",
	description	= L["FilterAll"],
}

Assessment.FilterType.NonZero =
{
	tag			= "BarValue ~= 0",
	description	= L["FilterNonZero"],
}

Assessment.ListType.FightDataList =
{
	method = function()
				return pairs(fightData)
			 end,
	events = { "Assessment:ResetFightData" },
	description = L["Fight Data List"],
}

Assessment.ListType.EnumerateTable =
{
	method = function(data, ...)
				for i=1,select('#',...) do
					if type(data) ~= "table" then return emptyMethod end
					local lookup = select(i, ...)
					data = data[lookup]
				end
				if type(data) ~= "table" then return emptyMethod end
				return pairs(data)
			 end,

	data = function(listTypeData)
				return Assessment.ListType.EnumerateTable.updateData[listTypeData]
		   end,
	
	description = L["Enumerate Data"],
	
	updateData = { }
}

--[[
local function AddToSparseDataTable(t, data, ...)
	for i=1,select('#', ...) do
		if type(data) ~= "table" then return end
		local lookup = select(i, ...)
		if lookup == "*" then
			for _,v in pairs(data) do
				AddToSparseDataTable(t, v, select(i+1, ...))
			end
			return
		else
			data = data[lookup]
		end
	end
	if data then
		t[#t+1] = data
	end
end

local function SparseTableIterator(state, key)
end

Assessment.ListType.EnumerateSparseTable =
{
	method = function(data, ...)
				local t = new()
				AddToSparseDataTable(t, data, ...)
				
				if #t == 0 then
					del(t)
					return emptyMethod
				end
				
				return SparseTableIterator, t
			 end,

	data = function(listTypeData)
				return Assessment.ListType.EnumerateTable.updateData[listTypeData]
		   end,
	
	description = L["Enumerate Sparse Data"],
}
]]--

-----------------------------------------------------------------------------

local playerName = UnitName("player")
Assessment.GatherFilterList =
{
	NONE    = { 
				method = function() return false end, 
				description = L["FILTER_NONE"]
			  },
	SELF    = { 
				method = function(name) return name == playerName end,
				description = L["FILTER_SELF"]
			  },
	ALL     = {
				method = function() return true end,
				description = L["FILTER_ALL"]
			  },
}
Assessment.GatherFilterValidate = { }

-----------------------------------------------------------------------------

local IterateParameterList
do
	local index
	
	local function ParameterListIterator(state)
		index = index + 1
		if index > #state then
			del(state)
		else
			return state[index]
		end
	end

	function IterateParameterList(state)
		if #state == 0 then
			del(state)
			return emptyMethod
		else
			index = 0
			return ParameterListIterator, state, nil
		end
	end
end

Assessment.ListType.EnumerateParameterList =
{
	method = function(data, ...)
				local state = new()
				for i=1,select('#',...) do
					state[i] = select(i, ...)
				end
				return IterateParameterList(state)
			 end,

	description = L["Enumerate Parameter List"],
}

-----------------------------------------------------------------------------

local function GetFightData(_, index)
	return fightData[index]
end

function Assessment:OnInitialize()
	profile = self.db.profile
	combatTime = self:GetModule("CombatTime")
	fightData = self.db.char.fightData

	for k,v in pairs(self.GatherFilterList) do
		self.GatherFilterValidate[k] = v.description
	end

	if not fightData[0] then
		fightData[0] = self:NewFightDataTable()
	end

	-- Install a GetFightData that actually does something
	self.GetFightData = GetFightData
end

function Assessment:OnEnable()
end

function Assessment:OnDisable()
end

-----------------------------------------------------------------------------

function Assessment:ResetFightData()
	eraseFightDataDirty = true

	for k,v in pairs(fightData) do
		fightData[k] = del(fightData[k], true)
	end
	fightData[0] = self:NewFightDataTable()
	self:TriggerEvent("Assessment:ResetFightData")
	
	self:Print(L["All fight data has been reset"])
end

function Assessment:ResetUnnamedFightData()
	for k,v in pairs(fightData) do
		if type(k) == "number" then
			fightData[k] = del(fightData[k], true)
		end
	end
	fightData[0] = self:NewFightDataTable()
	self:TriggerEvent("Assessment:ResetFightData")
	
	self:Print(L["Fight data has been reset"])
end

function Assessment:NewFightDataTable()
	local data = new()
	data.dataUpdateTimes = new()
	self:TriggerEvent("Assessment:NewFightDataTable", data)
	return data
end

function Assessment:CopyFightData(targetIndex, sourceIndex)
	eraseFightDataDirty = true
	if fightData[targetIndex] then
		fightData[targetIndex] = del(fightData[targetIndex], true)
	end
	
	if not fightData[sourceIndex] then return end
	
	local newFightData = copy(fightData[sourceIndex])
	fightData[targetIndex] = newFightData
	
	self:TriggerEvent("Assessment:CopyFightData", targetIndex, newFightData)
end

function Assessment:EraseFightData(key)
	if not fightData[key] then return end
	eraseFightDataDirty = true
	fightData[key] = del(fightData[key], true)
	self:TriggerEvent("Assessment:ResetFightData")
end

function Assessment:StartNewFight()
	if profile.pause then return end

	table.insert(fightData, 1, self:NewFightDataTable())
	
	if profile.history ~= 0 then
		for i=#fightData, profile.history+1, -1 do
			fightData[i] = del(fightData[i], true)
		end
	end

	self:TriggerEvent("Assessment:StartNewFight")
end

function Assessment:SetPause(value)
	if value == profile.pause then return end
	
	if value then
		self:TriggerEvent("Assessment:OnPauseStart")
		profile.pause = value
	else
		profile.pause = value
		self:TriggerEvent("Assessment:OnPauseStop")
	end
end

-----------------------------------------------------------------------------

-- Empty, replaced in OnInitialize
-- Some modules end up calling GetFightData, so it needs to be present, but
-- return nil until we're ready.
Assessment.GetFightData = emptyMethod

-----------------------------------------------------------------------------

-- These iterators are not to be called in a nested manner.
local iterateIndex = -1
local function DataTableIterator()
	if iterateIndex < 1 then
		iterateIndex = iterateIndex+1
		return fightData[iterateIndex]
	end
end

local function ForceIterateUpdateDataTables()
	iterateIndex = -1
	return DataTableIterator
end

local function IterateOOCUpdateDataTables()
	if profile.pause then
		return emptyMethod
	else 
		return ForceIterateUpdateDataTables()
	end
end

local function IterateUpdateDataTables()
	if profile.pause or (not profile.gatherOutOfCombat and not combatTime:IsInCombat()) then
		return emptyMethod
	else 
		return ForceIterateUpdateDataTables()
	end
end

-----------------------------------------------------------------------------

function Assessment:RedrawAll()
	self:TriggerEvent("Assessment:RedrawAll")
end

-----------------------------------------------------------------------------

Assessment.modulePrototype.IterateUpdateDataTables = IterateUpdateDataTables
Assessment.modulePrototype.IterateOOCUpdateDataTables = IterateOOCUpdateDataTables
Assessment.modulePrototype.ForceIterateUpdateDataTables = ForceIterateUpdateDataTables

function Assessment.modulePrototype.IterateFightData()
	return pairs(fightData)
end

function Assessment.modulePrototype.GetOptionValue(data, lookup)
	local v
	while lookup do
		v, lookup = ("."):split(lookup, 2)
		data = data[v]
	end
	return data
end

function Assessment.modulePrototype.SetOptionValue(data, lookup, value)
	local v
	while true do
		v, lookup = ("."):split(lookup, 2)		
		if lookup then
			data = data[v]
		else
			data[v] = value
			return
		end
	end
end

function Assessment.modulePrototype:CopyOptionsTemplate(source, i)
	local result = new()

	for k,v in pairs(source) do
		if k == "passValue" then
			local passValueTable = new()
			passValueTable[1] = i
			if type(v) == "table" then
				passValueTable[2] = copy(v)
			else
				passValueTable[2] = v
			end
			result[k] = passValueTable
		elseif type(v) == "table" then
			result[k] = self:CopyOptionsTemplate(v, i)
		else
			result[k] = v
		end
	end
	
	return result
end

function Assessment.modulePrototype:Debug(message)
	Assessment:Debug(message)
end

function Assessment.modulePrototype:CancelEvent(eventName)
	if self:IsEventRegistered(eventName) then
		self:UnregisterEvent(eventName)
	end
end

-----------------------------------------------------------------------------

StaticPopupDialogs["ASSESSMENT_SAVE_FIGHT_DATA"] = 
{
	text = L["Enter a name to save the fight data."],
	button1 = ACCEPT,
	button2 = CANCEL,
	hasEditBox = 1,
	OnAccept = function()
		local text = getglobal(this:GetParent():GetName().."EditBox"):GetText()
		if text ~= "" then
			Assessment:CopyFightData(text, StaticPopupDialogs["ASSESSMENT_SAVE_FIGHT_DATA"].data)
		end
	end,
	OnShow = function()
		getglobal(this:GetName().."EditBox"):SetText("")
		getglobal(this:GetName().."EditBox"):SetFocus()
	end,
	EditBoxOnEnterPressed = function()
		local text = getglobal(this:GetParent():GetName().."EditBox"):GetText()
		if text ~= "" then
			Assessment:CopyFightData(text, StaticPopupDialogs["ASSESSMENT_SAVE_FIGHT_DATA"].data)
		end
		this:GetParent():Hide()
	end,
	timeout = 0,
	exclusive = 1,
	whileDead = 1,
	hideOnEscape = 1,
}

-----------------------------------------------------------------------------
