﻿-----------------------------------------------------------------------------
--
-- data
--   \crowdControl
--     \<playerName>
--       - count
--       \detailList (array)
--         - time, targetName, abilityName
--   \crowdControlBreak
--     \<playerName>
--       - count
--       \detailList (array)
--         - time, targetName, crowdControlAbility, breakAbilityName
--
-----------------------------------------------------------------------------

local L = AceLibrary("AceLocale-2.2"):new("AssessmentCrowdControl")
L:RegisterTranslations("enUS", function() return {
	["Crowd Controllers List"] = true,
	["Crowd Control"] = true,
	["Crowd Control Breaks"] = true,
	["Crowd Control Details"] = true,
	["Crowd Control Break Details"] = true,

	["Crowd Control Options."] = true,
	["Detailed Data"] = true,
	["Gather detailed information on crowd control."] = true,

	["Break Data"] = true,
	["Gather detailed information on crowd control breaks."] = true,
	
	["Melee"] = true,
} end)

L:RegisterTranslations("koKR", function() return {
	["Crowd Controllers List"] = "군중 제어 리스트",
	["Crowd Control"] = "군중 제어",
	["Crowd Control Details"] = "군중 제어 상세 내용",

	["Crowd Control Options."] = "군중 제어 설정",
	["Detailed Data"] = "상세 데이터 수집",
	["Gather detailed information on crowd control."] = "군중 제어에 관한 상세 정보를 수집합니다.",

	["Break Data"] = "수집 중단",
	["Gather detailed information on crowd control breaks."] = "군중 제어에 관한 정보 수집을 중단합니다.",
	
	["Melee"] = "근접",
} end)

local BS = AceLibrary("Babble-Spell-2.2")

-----------------------------------------------------------------------------

local AssessmentCrowdControl = Assessment:NewModule("CrowdControl", "AceEvent-2.0", "Parser-3.0")
AssessmentCrowdControl.db = Assessment:AcquireDBNamespace("CrowdControl")
Assessment:RegisterDefaults("CrowdControl", "profile", {
	enable						= true,
	gatherDetailDataFilter		= "GROUP",
	gatherBreakDetailDataFilter	= "GROUP",
})

-----------------------------------------------------------------------------

local profile
local gatherDetailDataMethod
local gatherBreakDetailDataMethod

local emptyMethod = Assessment.EmptyMethod
local new = Assessment.NewTable
local pairs = pairs

-----------------------------------------------------------------------------

Assessment.TagData.CrowdControlCount =
{
	method = function(data)
				local fightData = data.fightData
				if not fightData then return 0 end
				
				local crowdControl = fightData.crowdControl
				if not crowdControl then return 0 end
				
				local playerCrowdControl = crowdControl[data.key]
				if not playerCrowdControl then return 0 end

				return playerCrowdControl.count
			 end,
	data = "crowdControl",
	guaranteeResult = true,
	numeric = true,
}

Assessment.TagData.CrowdControlBreakCount =
{
	method = function(data)
				local fightData = data.fightData
				if not fightData then return 0 end
				
				local crowdControlBreak = fightData.crowdControlBreak
				if not crowdControlBreak then return 0 end
				
				local playerCrowdControl = crowdControlBreak[data.key]
				if not playerCrowdControl then return 0 end

				return playerCrowdControl.count
			 end,
	data = "crowdControlBreak",
	guaranteeResult = true,
	numeric = true,
}

-----------------------------------------------------------------------------

Assessment.ListType.CrowdControllers =
{
	method = function(fightData)
				if not fightData.crowdControl then return emptyMethod end
				return pairs(fightData.crowdControl)
			 end,
	data = "crowdControl",
	description = L["Crowd Controllers List"],
}

-----------------------------------------------------------------------------

Assessment.DefaultDisplaySets.CrowdControlDetails =
{
	titleTag			= ("%s: [WindowParameter(1)]"):format(L["Crowd Control"]),
	listType			= "EnumerateTable",
	listDataTag			= "crowdControl,[WindowParameter(1)],detailList",
	barValueTag			= "0",
	filterTag			= "1",
	fightIndex			= -1,
	text				= { count = 2, { tag = "[GetValueField(\"targetName\")] [GetValueField(\"abilityName\"):Bracket]" }, { tag = "[GetValueField(\"time\"):FormatTime]" } },
	description			= L["Crowd Control Details"],
	selectable			= false,
	defaultColor		= { 0.3, 0.3, 0.8 },
	favorite			= false,
}

Assessment.DefaultDisplaySets.CrowdControl =
{
	titleTag			= L["Crowd Control"],
	listType			= "GroupMembers", 
	barValueTag			= "CrowdControlCount", 
	filterTag			= "BarValue ~= 0",
	fightIndex			= 0,
	description			= L["Crowd Control"],
	defaultColor		= { 0.3, 0.3, 0.8 },
	colorTypeList		= { "Class", count = 1 },
	onClickType			= "SHOW_DISPLAY_SET",
	onClickDisplaySet	= L["Crowd Control Details"],
	onClickData			= "[Key]",
	favorite			= false,
}

Assessment.DefaultDisplaySets.CrowdControlBreakDetails =
{
	titleTag			= ("%s: [WindowParameter(1)]"):format(L["Crowd Control Breaks"]),
	listType			= "EnumerateTable",
	listDataTag			= "crowdControlBreak,[WindowParameter(1)],detailList",
	barValueTag			= "0",
	filterTag			= "1",
	fightIndex			= -1,
	text				= { count = 2, { tag = "[GetValueField(\"targetName\")] [GetValueField(\"crowdControlAbility\"):Bracket]" }, { tag = "[GetValueField(\"time\"):FormatTime]" } },
	description			= L["Crowd Control Break Details"],
	selectable			= false,
	defaultColor		= { 0.3, 0.3, 0.8 },
	favorite			= false,
}

Assessment.DefaultDisplaySets.CrowdControlBreaks =
{
	titleTag			= L["Crowd Control Breaks"],
	listType			= "GroupMembers", 
	barValueTag			= "CrowdControlBreakCount", 
	filterTag			= "BarValue ~= 0",
	fightIndex			= 0,
	description			= L["Crowd Control Breaks"],
	defaultColor		= { 0.3, 0.3, 0.8 },
	colorTypeList		= { "Class", count = 1 },
	onClickType			= "SHOW_DISPLAY_SET",
	onClickDisplaySet	= L["Crowd Control Break Details"],
	onClickData			= "[Key]",
}

-----------------------------------------------------------------------------

Assessment.ListType.EnumerateTable.updateData.crowdControl = "crowdControl"
Assessment.ListType.EnumerateTable.updateData.crowdControlBreak = "crowdControlBreak"

-----------------------------------------------------------------------------

Assessment.Options.args.data.args.crowdControl =
{
	type = "group",
	name = L["Crowd Control"],
	desc = L["Crowd Control Options."],
	args =
	{
		gatherDetailsData =
		{
			type = "text",
			name = L["Detailed Data"],
			desc = L["Gather detailed information on crowd control."],
			get = function() return profile.gatherDetailDataFilter end,
			set = function(v) profile.gatherDetailDataFilter = v AssessmentCrowdControl:OnInitialize() end,
			validate = Assessment.GatherFilterValidate,
		},
		gatherBreakDetailsData =
		{
			type = "text",
			name = L["Break Data"],
			desc = L["Gather detailed information on crowd control breaks."],
			get = function() return profile.gatherBreakDetailDataFilter end,
			set = function(v) profile.gatherBreakDetailDataFilter = v AssessmentCrowdControl:OnInitialize() end,
			validate = Assessment.GatherFilterValidate,
		},
	}
}

-----------------------------------------------------------------------------

function AssessmentCrowdControl:OnInitialize()
	profile = self.db.profile
	gatherDetailDataMethod = Assessment.GatherFilterList[profile.gatherDetailDataFilter].method
	gatherBreakDetailDataMethod = Assessment.GatherFilterList[profile.gatherBreakDetailDataFilter].method
end

function AssessmentCrowdControl:OnEnable()
	self:RegisterParserEvent({ eventType = "Cast" }, "OnParserCastEvent")
	self:RegisterParserEvent({ eventType = "Dispel" }, "OnParserDispelEvent")
end

function AssessmentCrowdControl:OnDisable()
end

-----------------------------------------------------------------------------

local crowdControlSpells =
{
	[BS["Freezing Trap Effect"]]	= true,
	[BS["Hibernate"]]				= true,
	[BS["Polymorph"]]				= true,
	[BS["Polymorph: Pig"]]			= true,
	[BS["Polymorph: Turtle"]]		= true,
	[BS["Sap"]]						= true,
	[BS["Seduction"]]				= true,
	[BS["Shackle Undead"]]			= true,
}

-----------------------------------------------------------------------------

function AssessmentCrowdControl:OnParserCastEvent(eventData)
	if eventData.isBegin then return end

	local abilityName = eventData.abilityName
	if not crowdControlSpells[abilityName] then return end

	local sourceName = eventData.sourceName
	if not sourceName then return end

	local recipientName = eventData.recipientName
	if not recipientName then return end
	
	local crowdControlTime = time()
	local time = GetTime()

	for data in self:IterateOOCUpdateDataTables() do
		data.dataUpdateTimes.crowdControl = time
		local crowdControl = data.crowdControl
		if not crowdControl then
			crowdControl = new()
			data.crowdControl = crowdControl
		end

		local playerCrowdControl = crowdControl[sourceName]
		if not playerCrowdControl then
			playerCrowdControl = new()
			playerCrowdControl.count = 1
			crowdControl[sourceName] = playerCrowdControl
		else
			playerCrowdControl.count = playerCrowdControl.count + 1
		end
		
		if gatherDetailDataMethod(sourceName) then
			local detailList = playerCrowdControl.detailList
			if not detailList then
				detailList = new()
				playerCrowdControl.detailList = detailList
			end

			local details = new()
			details.time = crowdControlTime
			details.targetName = recipientName
			details.abilityName = abilityName
			
			detailList[#detailList+1] = details
		end
	end
end

local meleeText = L["Melee"]
function AssessmentCrowdControl:OnParserDispelEvent(eventData)
	local sourceName = eventData.sourceName
	if not sourceName then return end

	local recipientName = eventData.recipientName
	
	local recipientAbilityName = eventData.recipientAbilityName
	if not crowdControlSpells[recipientAbilityName] then return end

	local crowdControlBreakTime = time()
	local time = GetTime()

	for data in self:IterateOOCUpdateDataTables() do
		data.dataUpdateTimes.crowdControlBreak = time
		local crowdControlBreak = data.crowdControlBreak
		if not crowdControlBreak then
			crowdControlBreak = new()
			data.crowdControlBreak = crowdControlBreak
		end

		local playerCrowdControl = crowdControlBreak[sourceName]
		if not playerCrowdControl then
			playerCrowdControl = new()
			playerCrowdControl.count = 1
			crowdControlBreak[sourceName] = playerCrowdControl
		else
			playerCrowdControl.count = playerCrowdControl.count + 1
		end
		
		if gatherBreakDetailDataMethod(sourceName) then
			local detailList = playerCrowdControl.detailList
			if not detailList then
				detailList = new()
				playerCrowdControl.detailList = detailList
			end

			local details = new()
			details.time = crowdControlBreakTime
			details.targetName = recipientName
			details.crowdControlAbility = recipientAbilityName
			details.breakAbilityName = eventData.sourceAbilityName or meleeText
			
			detailList[#detailList+1] = details
		end
	end
end

-----------------------------------------------------------------------------
