﻿-----------------------------------------------------------------------------
--
-- data
--   \interrupt
--     \<playerName>
--       - count
--       \detailList (array)
--         - time, targetName, abilityName
--
-----------------------------------------------------------------------------

local L = AceLibrary("AceLocale-2.2"):new("AssessmentInterrupt")
L:RegisterTranslations("enUS", function() return {
	["Interrupt List"] = true,
	["Interrupts"] = true,
	["Interrupt Details"] = true,
	["Melee"] = true,

	["Interrupt"] = true,
	["Interrupt Options."] = true,
	["Detail Data"] = true,
	["Gather information on each interrupt used."] = true,
} end)

L:RegisterTranslations("koKR", function() return {
	["Interrupt List"] = "중단 리스트",
	["Interrupts"] = "중단",
	["Interrupt Details"] = "중단 상세 정보",
	["Melee"] = "근접",

	["Interrupt"] = "중단",
	["Interrupt Options."] = "중단 설정",
	["Detail Data"] = "상세 정보 수집",
	["Gather information on each interrupt used."] = "중단에 대한 정보를 수집합니다.",
} end)

-----------------------------------------------------------------------------

local AssessmentInterrupt = Assessment:NewModule("Interrupt", "AceEvent-2.0", "Parser-3.0")
AssessmentInterrupt.db = Assessment:AcquireDBNamespace("Interrupt")
Assessment:RegisterDefaults("Interrupt", "profile", {
	enable				= true,
	gatherDetailFilter	= "ALL",
})

-----------------------------------------------------------------------------

local profile
local gatherDetailDataMethod

local emptyMethod = Assessment.EmptyMethod
local new = Assessment.NewTable
local pairs = pairs

-----------------------------------------------------------------------------

Assessment.TagData.InterruptCount =
{
	method = function(data)
				local fightData = data.fightData
				if not fightData then return 0 end
				
				local interrupt = fightData.interrupt
				if not interrupt then return 0 end
				
				local playerInterrupt = interrupt[data.key]
				if not playerInterrupt then return 0 end

				return playerInterrupt.count
			 end,
	data = "interrupt",
	guaranteeResult = true,
	numeric = true,
}

-----------------------------------------------------------------------------

Assessment.ListType.Interrupts =
{
	method = function(fightData)
				if not fightData.interrupt then return emptyMethod end
				return pairs(fightData.interrupt)
			 end,
	data = "interrupt",
	description = L["Interrupt List"],
}

-----------------------------------------------------------------------------

Assessment.DefaultDisplaySets.InterruptDetails =
{
	titleTag			= ("%s: [WindowParameter(1)]"):format(L["Interrupt Details"]),
	listType			= "EnumerateTable",
	listDataTag			= "interrupt,[WindowParameter(1)],detailList",
	barValueTag			= "0",
	filterTag			= "1",
	fightIndex			= -1,
	text				= { count = 2, { tag = "[GetValueField(\"abilityName\")] [GetValueField(\"targetName\"):Bracket]" }, { tag = "[GetValueField(\"time\"):FormatTime]" } },
	description			= L["Interrupt Details"],
	selectable			= false,
	defaultColor		= { 0.8, 0.8, 0.5 },
	favorite			= false,
}

Assessment.DefaultDisplaySets.Interrupts =
{
	titleTag			= L["Interrupts"],
	listType			= "GroupMembers", 
	barValueTag			= "InterruptCount", 
	filterTag			= "BarValue ~= 0",
	fightIndex			= 1,
	description			= L["Interrupts"],
	defaultColor		= { 0.8, 0.8, 0.5 },
	colorTypeList		= { "Class", count = 1 },
	onClickType			= "SHOW_DISPLAY_SET",
	onClickDisplaySet	= L["Interrupt Details"],
	onClickData			= "[Key]",
}

-----------------------------------------------------------------------------

Assessment.ListType.EnumerateTable.updateData.interupt = "interrupt"

-----------------------------------------------------------------------------

Assessment.Options.args.data.args.interrupt =
{
	type = "group",
	name = L["Interrupt"],
	desc = L["Interrupt Options."],
	args =
	{
		gatherTimeData =
		{
			type = "text",
			name = L["Detail Data"],
			desc = L["Gather information on each interrupt used."],
			get = function() return profile.gatherDetailFilter end,
			set = function(v) profile.gatherDetailFilter = v AssessmentInterrupt:OnInitialize() end,
			validate = Assessment.GatherFilterValidate,
		},
	}
}

-----------------------------------------------------------------------------

function AssessmentInterrupt:OnInitialize()
	profile = self.db.profile
	gatherDetailDataMethod = Assessment.GatherFilterList[profile.gatherDetailFilter].method
end

function AssessmentInterrupt:OnEnable()
	self:RegisterParserEvent({ eventType = "Interrupt" }, "OnParserInterruptEvent")
end

function AssessmentInterrupt:OnDisable()
end

-----------------------------------------------------------------------------

local meleeText = L["Melee"]
function AssessmentInterrupt:OnParserInterruptEvent(eventData)
	local interruptTime = time()
	local time = GetTime()

	for data in self:IterateUpdateDataTables() do
		data.dataUpdateTimes.interrupt = time
		local interrupt = data.interrupt
		if not interrupt then
			interrupt = new()
			data.interrupt = interrupt
		end

		local sourceName = eventData.sourceName
		local playerInterrupt = interrupt[sourceName]
		if playerInterrupt then
			playerInterrupt.count = playerInterrupt.count + 1
		else
			playerInterrupt = new()
			playerInterrupt.count = 1
			interrupt[sourceName] = playerInterrupt
		end

		if gatherDetailDataMethod(sourceName) then
			local detailList = playerInterrupt.detailList
			if not detailList then
				detailList = new()
				playerInterrupt.detailList = detailList
			end

			local details = new()
			details.time = interruptTime
			details.targetName = eventData.recipientName
			details.abilityName = eventData.abilityName or meleeText
			detailList[#detailList+1] = details
		end
	end
end

-----------------------------------------------------------------------------
